@@ -7,9 +7,11 @@ Stores the class for SkewTDisplay.
 import matplotlib.pyplot as plt
 import numpy as np
 import warnings
+import scipy
 
 try:
     from pkg_resources import DistributionNotFound
+    import metpy
     import metpy.calc as mpcalc
     METPY_AVAILABLE = True
 except ImportError:
@@ -328,7 +330,15 @@ class SkewTDisplay(Display):
         self.SkewT[subplot_index].plot_barbs(
             p_levels_to_plot, u_red, v_red, **plot_barbs_kwargs)
 
-        prof = mpcalc.parcel_profile(p, T[0], Td[0]).to('degC')
+        # Metpy fix if Pressure does not decrease monotonically in
+        # your sounding.
+        try:
+            prof = mpcalc.parcel_profile(p, T[0], Td[0]).to('degC')
+        except metpy.calc.exceptions.InvalidSoundingError:
+            p = scipy.ndimage.median_filter(p, 3, output=float)
+            p = metpy.units.units.Quantity(p, p_units)
+            prof = mpcalc.parcel_profile(p, T[0], Td[0]).to('degC')
+
         if show_parcel:
             # Only plot where prof > T
             lcl_pressure, lcl_temperature = mpcalc.lcl(p[0], T[0], Td[0])
