@@ -4,11 +4,13 @@ Stores the class for TimeSeriesDisplay.
 """
 
 import matplotlib.pyplot as plt
+import matplotlib.dates as mdates
 import numpy as np
 import pandas as pd
 import datetime as dt
 import warnings
 
+from re import search
 from re import search as re_search
 from matplotlib import colors as mplcolors
 from mpl_toolkits.axes_grid1 import make_axes_locatable
@@ -416,7 +418,10 @@ class TimeSeriesDisplay(Display):
                     data, abs_limits[1])
 
             # Plot the data
-            lines = ax.plot(xdata, data, '.', **kwargs)
+            if 'marker' not in kwargs.keys():
+                kwargs['marker'] = '.'
+
+            lines = ax.plot(xdata, data, **kwargs)
 
             # Check if we need to call legend method after plotting. This is only
             # called when no assessment overplot is called.
@@ -478,15 +483,23 @@ class TimeSeriesDisplay(Display):
             # Sets shading parameter to auto. Matplotlib will check deminsions.
             # If X,Y and C are same deminsions shading is set to nearest.
             # If X and Y deminsions are 1 greater than C shading is set to flat.
+            if 'edgecolors' not in kwargs.keys():
+                kwargs['edgecolors'] = 'face'
             mesh = ax.pcolormesh(np.asarray(xdata), ydata, data.transpose(),
-                                 shading=set_shading, cmap=cmap, edgecolors='face',
-                                 **kwargs)
+                                 shading=set_shading, cmap=cmap, **kwargs)
 
         # Set Title
         if set_title is None:
-            set_title = ' '.join([dsname, field, 'on',
-                                 dt_utils.numpy_to_arm_date(
-                                     self._obj[dsname].time.values[0])])
+            if isinstance(self._obj[dsname].time.values[0], np.datetime64):
+                set_title = ' '.join([dsname, field, 'on',
+                                     dt_utils.numpy_to_arm_date(
+                                         self._obj[dsname].time.values[0])])
+            else:
+                date_result = search(r"\d{4}-\d{1,2}-\d{1,2}", self._obj[dsname].time.attrs['units'])
+                if date_result is not None:
+                    set_title = ' '.join([dsname, field, 'on', date_result.group(0)])
+                else:
+                    set_title = ' '.join([dsname, field])
 
         if secondary_y is False:
             ax.set_title(set_title)
@@ -529,6 +542,9 @@ class TimeSeriesDisplay(Display):
 
             # Check if current range is outside of new range an only set
             # values that work for all data plotted.
+            if isinstance(yrng[0], np.datetime64):
+                yrng = mdates.datestr2num([str(yrng[0]), str(yrng[1])])
+
             current_yrng = ax.get_ylim()
             if yrng[0] > current_yrng[0]:
                 yrng[0] = current_yrng[0]
@@ -1123,6 +1139,8 @@ class TimeSeriesDisplay(Display):
         assessment_color : dict
             Dictionary lookup to override default assessment to color. Make sure
             assessment work is correctly set with case syntax.
+        edgecolor : str or list
+            Color name, list of color names or 'face' as defined in matplotlib.axes.Axes.broken_barh
         set_shading : string
             Option to to set the matplotlib.pcolormesh shading parameter.
             Default to 'auto'
@@ -1290,6 +1308,7 @@ class TimeSeriesDisplay(Display):
                 # Plot green data first.
                 ax.broken_barh(barh_list_green, (ii, ii + 1), facecolors=color_lookup['Not Failing'],
                                edgecolor=edgecolor, **kwargs)
+
                 # Get test number from flag_mask bitpacked number
                 test_nums.append(parse_bit(flag_masks[ii]))
                 # Get masked array data to use mask for finding if/where test is set
@@ -1307,8 +1326,7 @@ class TimeSeriesDisplay(Display):
                             assess = "Missing"
                             break
                     # Lay down blocks of tripped tests using correct color
-                    ax.broken_barh(barh_list, (ii, ii + 1),
-                                   facecolors=color_lookup[assess],
+                    ax.broken_barh(barh_list, (ii, ii + 1), facecolors=color_lookup[assess],
                                    edgecolor=edgecolor, **kwargs)
 
                 # Add test description to plot.
