@@ -26,7 +26,7 @@ class QCFilter(qctests.QCTests, comparison_tests.QCTests, object):
         self._obj = xarray_obj
 
     def check_for_ancillary_qc(self, var_name, add_if_missing=True,
-                               cleanup=True, flag_type=False):
+                               cleanup=False, flag_type=False):
         """
         Method to check if a quality control variable exist in the dataset
         and return the quality control varible name.
@@ -92,8 +92,7 @@ class QCFilter(qctests.QCTests, comparison_tests.QCTests, object):
             self._obj.qcfilter.update_ancillary_variable(var_name, qc_var_name)
 
         # Clean up quality control variables to the requried standard in the
-        # xarray object. If the quality control variables are already cleaned
-        # the extra work is small since it's just checking.
+        # xarray object.
         if cleanup:
             self._obj.clean.cleanup(handle_missing_value=True,
                                     link_qc_variables=False)
@@ -336,7 +335,7 @@ class QCFilter(qctests.QCTests, comparison_tests.QCTests, object):
 
         return test_dict
 
-    def remove_test(self, var_name=None, qc_var_name=None, test_number=None, flag_value=False,
+    def remove_test(self, var_name=None, test_number=None, qc_var_name=None, flag_value=False,
                     flag_values_reset_value=0):
         """
         Method to remove a test/filter from a quality control variable. Must set
@@ -346,10 +345,10 @@ class QCFilter(qctests.QCTests, comparison_tests.QCTests, object):
         ----------
         var_name : str or None
             Data variable name.
-        qc_var_name : str or None
-            Quality control variable name. Ignored if var_name is set.
         test_number : int
             Test number to remove.
+        qc_var_name : str or None
+            Quality control variable name. Ignored if var_name is set.
         flag_value : boolean
             Switch to use flag_values integer quality control.
         flag_values_reset_value : int
@@ -363,11 +362,11 @@ class QCFilter(qctests.QCTests, comparison_tests.QCTests, object):
         """
         if test_number is None:
             raise ValueError('You need to provide a value for test_number '
-                             'keyword when calling the add_test() method')
+                             'keyword when calling the remove_test() method')
 
         if var_name is None and qc_var_name is None:
             raise ValueError('You need to provide a value for var_name or qc_var_name '
-                             'keyword when calling the add_test() method')
+                             'keyword when calling the remove_test() method')
 
         if var_name is not None:
             qc_var_name = self._obj.qcfilter.check_for_ancillary_qc(var_name)
